local Lib = MissionMinderPreLoad;
local DEBUG = MissionMinderPreLoad.debug;


MissionMinderPreLoadGlobals = {};
local MissionMinderPreLoadGlobals_DEFAULT = {
	["Characters"] = {},
};

local autocreate = {
	__index = function(self, key) -- Create empty table for previously unknown entries
		rawset(self, key, {})
		return self[key]
	end
}

-- Try to determine retroactively if a character hasGarrison
local function calculateHasGarrison(details)
	details.hasGarrison = false

	-- Garrison Cache
	if ( not details.hasGarrison and details.GarrisonCache ) then
		details.hasGarrison = true
	end

	-- GarrisonResources
	if ( not details.hasGarrison and details.GarrsionResources ) then
		details.hasGarrison = details.GarrsionResources > 0
	end

	-- MaxGarrisonResources
	if ( not details.hasGarrison and details.MaxGarrisonResources ) then
		details.hasGarrison = details.MaxGarrisonResources > 0
	end

	-- Work Orders
	if ( not details.hasGarrison and details.workorders ) then
		details.hasGarrison = #details.workorders > 0;
	end

	-- Missions
	if ( not details.hasGarrison and details.missions ) then
		local count = 0
		for _,_ in pairs(details.missions) do
			count = count + 1
		end
		details.hasGarrison = count > 0;
	end

	-- Buildings
	if ( not details.hasGarrison and details.buildings ) then
		local count = 0
		for _,_ in pairs(details.buildings) do
			count = count + 1
		end
		details.hasGarrison = count > 0;
	end
end


local function initialize()
	DEBUG("   |cffffffffpreload-Options|r initialized");


	-- SavedVariables have been loaded, rewrite to include any new or missing values
	for key,value in pairs(MissionMinderPreLoadGlobals_DEFAULT) do
		if ( MissionMinderPreLoadGlobals[key] == nil ) then
			MissionMinderPreLoadGlobals[key] = value;
		end
	end

	-- Set Globals.Characters to automatically create new entries
	MissionMinderPreLoadGlobals.Characters = setmetatable(MissionMinderPreLoadGlobals.Characters, autocreate)

	-- hasGarrison migration
	for character, details in pairs(MissionMinderPreLoadGlobals.Characters) do
		if ( details.hasGarrison == nil ) then
			calculateHasGarrison(details)
		end
	end

	-- GarrsionResources typo migration
	for character, details in pairs(MissionMinderPreLoadGlobals.Characters) do
		if ( details.GarrsionResources ) then
			details.GarrisonResources = details.GarrsionResources -- copy data
			details.GarrsionResources = nil -- remove bad
		end
	end


	-- Remove missions that are not tables
	for _, character in pairs(MissionMinderPreLoadGlobals.Characters) do
		if ( character.missions ) then
			for missionID, mission in pairs(character.missions) do
				if ( type(mission) ~= "table") then
					character.missions[missionID] = nil
				end
			end
		end
	end

	-- Rewrite to remove any deprecated settings
	for key,value in pairs(MissionMinderPreLoadGlobals) do
		if ( MissionMinderPreLoadGlobals_DEFAULT[key] == nil ) then
			MissionMinderPreLoadGlobals[key] = nil;
		end
	end

	Lib.Globals = MissionMinderPreLoadGlobals;
	Lib.callbacks:Fire("VARIABLES_LOADED")
end

Lib:RegisterEvent("options", "ADDON_LOADED", initialize);
