local Lib = MissionMinder
Lib.callbacks = Lib.callbacks or LibStub("CallbackHandler-1.0"):New(Lib)

local OPTIONS = Lib.options;
local DEBUG = Lib.debug;
local module = Lib.multiplanner
local states = module.states

local L = LibStub("AceLocale-3.0"):GetLocale("MissionMinder");

local COLLAPSED = states.BASE:new("COLLAPSED")
states.COLLAPSED = COLLAPSED

function COLLAPSED:new()
	local obj = {}
	setmetatable(obj, self)
	return obj
end

local MINUTES = 60;
local HOURS = 60 * MINUTES;
local DAYS = 24 * HOURS;

local function getCompactDurationString(time)
	local str;
	if (time >= DAYS) then
		time = ((time - 1) / DAYS) + 1;
		return string.format(COOLDOWN_DURATION_DAYS, time);
	elseif (time >= HOURS) then
		time = ((time - 1) / HOURS) + 1;
		return string.format(COOLDOWN_DURATION_HOURS, time);
	elseif (time >= MINUTES) then
		time = ((time - 1) / MINUTES) + 1;
		return string.format(COOLDOWN_DURATION_MIN, time);
	else
		return string.format(COOLDOWN_DURATION_SEC, time);
	end
end

local function isEmpty(list, size, truevalue, falsevalue)
	list = list or {}
	size = size or #list
	truevalue = truevalue or true
	falsevalue = falsevalue or false

	for i = 1, size do
		if (list[i]) then
			return falsevalue
		end
	end

	return truevalue
end

local function adjustForCollapsedDetails(button)
	local listscroll = GarrisonMissionFrame.MultiPlanner.MissionsList
	button:SetHeight(listscroll.collapsedButtonHeight)

	button.IconBG:SetPoint("BOTTOMLEFT", button.ButtonRegion, 0, 1)
	button.Header:Hide()
	button.PartyBuffs:Hide()
	button.SelectT:Hide()
	button.SelectB:Hide()
	button.SelectTL:Hide()
	button.SelectTR:Hide()
	button.SelectBL:Hide()
	button.SelectBR:Hide()
	button.LocBack:Hide()
	button.LocMid:Hide()
	button.LocFore:Hide()
	button.LocBottomBlend:Hide()
	button.MissionTime:Hide()
	button.MissionXP:Hide()
	for i = 1, #button.Followers do
		button.Followers[i]:Hide()
	end
	for i = 1, #button.Enemies do
		button.Enemies[i]:Hide()
	end

	button.Rewards[1]:ClearAllPoints()
	button.Rewards[1]:SetPoint("BOTTOMRIGHT", -9, 5)
	for i = 1, #button.Rewards do
		button.Rewards[i]:SetScale(0.8)
	end
end

local function updateCollapsedMissionMechanics(button)
	local mission = button.info

	local location, xp, environment, environmentDesc, environmentTexture, locPrefix, isExhausting, enemies = C_Garrison.GetMissionInfo(mission.missionID);
	button.MissionEnvIcon.Texture:SetTexture(environmentTexture);
	button.MissionEnvIcon.environment = environment
	button.MissionEnvIcon.environmentDesc = environmentDesc
	button.MissionEnvIcon:Show()

	local numMechs = 0
	for i = 1, #enemies do
		local enemy = enemies[i]
		for id, mechanic in pairs(enemy.mechanics) do
			numMechs = numMechs + 1;
			if (not button.Mechanics[numMechs]) then
				button.Mechanics[numMechs] = CreateFrame("Button", nil, button, "PlannerMechanicTemplate");
				button.Mechanics[numMechs]:SetPoint("RIGHT", button.Mechanics[numMechs-1], "LEFT", -2, 0);
			end
			local Mechanic = button.Mechanics[numMechs];
			Mechanic.info = mechanic;
			Mechanic.mechanicID = id;
			Mechanic.Texture:SetTexture(mechanic.icon);
			Mechanic:Show();
		end
	end
	for j=(numMechs + 1), #button.Mechanics do
		button.Mechanics[j]:Hide();
		button.Mechanics[j].mechanicID = nil;
		button.Mechanics[j].info = nil;
	end
end

local function GarrisonMissionPlanner_CheckCounter(button, counterID)
	for i = 1, #button.Mechanics do
		if ( counterID == button.Mechanics[i].mechanicID and not button.Mechanics[i].hasCounter ) then
			button.Mechanics[i].hasCounter = true;
			button.Mechanics[i].Check:Show();
			return;
		end
	end
end

local function updateCollapsedMissionForParty(button)
	local mission = button.info
	if ( not mission ) then
		return
	end

	local location, xp, environment, environmentDesc, environmentTexture, locPrefix, isExhausting, enemies = C_Garrison.GetMissionInfo(mission.missionID);
	local totalTimeString, totalTimeSeconds, isMissionTimeImproved, successChance, partyBuffs, isEnvMechanicCountered, xpBonus, materialMultiplier, goldMultiplier = C_Garrison.GetPartyMissionInfo(mission.missionID)

	-- TIME
	if ( isMissionTimeImproved ) then
		totalTimeString = GREEN_FONT_COLOR_CODE..totalTimeString..FONT_COLOR_CODE_CLOSE
	elseif ( totalTimeSeconds >= GARRISON_LONG_MISSION_TIME ) then
		totalTimeString = format(GARRISON_LONG_MISSION_TIME_FORMAT, totalTimeString)
	end
	button.Summary:SetFormattedText(PARENS_TEMPLATE, totalTimeString)
	button.MissionTime:SetFormattedText(GARRISON_MISSION_TIME_TOTAL, totalTimeString)

	-- SUCCESS CHANCE
	button.Chance:SetFormattedText(PERCENTAGE_STRING, successChance)

	-- ENVIRONMENT
	if ( isEnvMechanicCountered ) then
		button.MissionEnvIcon.Check:Show()
	else
		button.MissionEnvIcon.Check:Hide()
	end

	-- MECHANICS
	for i = 1, #button.Mechanics do
		button.Mechanics[i].hasCounter = nil
		button.Mechanics[i].Check:Hide()
	end
	for i = 1, #button.MinimizedFollowers do
		local frame = button.MinimizedFollowers[i]
		local follower = frame.info
		if ( follower ) then
			if (C_Garrison.GetFollowerBiasForMission(mission.missionID, follower.followerID) > -1 ) then
				if (not follower.abilities) then
					follower.abilities = C_Garrison.GetFollowerAbilities(follower.followerID)
				end
				for a = 1, #follower.abilities do
					local ability = follower.abilities[a];
					for counterID, counterInfo in pairs(ability.counters) do
						GarrisonMissionPlanner_CheckCounter(button, counterID)
					end
				end
			end
		end
	end

	-- Rewards
	GarrisonMissionPage_UpdateRewardQuantities(button, materialMultiplier, goldMultiplier);
end

function COLLAPSED:getFollowerFrames(button)
	return button.MinimizedFollowers
end

function COLLAPSED:missionEnter()
end

function COLLAPSED:missionClick()
end

function COLLAPSED:placerClick(placer, button)
	local buttons = GarrisonMissionFrame.MultiPlanner.MissionsList.buttons;
	local followers = button.MinimizedFollowers

	for i = 1, #followers do
		local frame = followers[i];
		if ( frame:IsShown() and frame:IsMouseOver() ) then
			GarrisonMissionPlannerMinimizedFollower_OnReceiveDrag(frame);
		end
	end
end

function COLLAPSED:animateButton(button, elapsed)
end

function COLLAPSED:updateMissionDetails(button)
	local parties = module.parties
	local mission = button.info
	if ( mission and self.dirty ) then
		adjustForCollapsedDetails(button)

		-- Duration
		button.Summary:Show()
		if ( mission.durationSeconds >= GARRISON_LONG_MISSION_TIME ) then
			local duration = format(GARRISON_LONG_MISSION_TIME_FORMAT, mission.duration);
			button.Summary:SetFormattedText(PARENS_TEMPLATE, duration);
		else
			button.Summary:SetFormattedText(PARENS_TEMPLATE, mission.duration);
		end

		-- Title
		button.Title:SetWidth(0);
		button.Title:SetText(mission.name);
		button.Title:SetPoint("LEFT", 165, 0);

		-- Location
		if ( mission.locPrefix ) then
			button.LocBG:Show();
			button.LocBG:SetAtlas(mission.locPrefix.."-List");
		else
			button.LocBG:Hide();
		end

		-- Rare Mission
		if (mission.isRare) then
			button.RareOverlay:Show();
			button.RareText:Show();
			button.IconBG:SetVertexColor(0, 0.012, 0.291, 0.5)
		else
			button.RareOverlay:Hide();
			button.RareText:Hide();
			button.IconBG:SetVertexColor(0, 0, 0, 0.5)
		end

		-- Mission Type
		button.Icon:SetAtlas(mission.typeAtlas)

		-- Mission Level
		local showingItemLevel = false;
		if ( mission.level == GARRISON_FOLLOWER_MAX_LEVEL and mission.iLevel > 0 ) then
			showingItemLevel = true;
		end
		button.Level:SetText(showingItemLevel and mission.iLevel or mission.level);

		-- Success Chance
		button.Chance:SetPoint("CENTER", button.ButtonRegion, "LEFT", 40, (mission.isRare) and -2 or -10);
		local successChanceColor = isEmpty(parties[mission.missionID], mission.numFollowers, module.SUCCESS_CHANCE_DEFAULT_COLOR, module.SUCCESS_CHANCE_IN_USE_COLOR)
		button.Chance:SetTextColor(unpack(successChanceColor))

		-- Expires
		if ( mission.offerEndTime ) then
			local timeRemaining = mission.offerEndTime - GetTime();
			if ( timeRemaining > 0 ) then
				button.Expires:SetText(getCompactDurationString(timeRemaining))
				if ( timeRemaining < 2*HOURS ) then
					button.Expires:SetTextColor(1, 0, 0);
				else
					button.Expires:SetTextColor(1, 1, 1);
				end
				button.Expires:Show();
			else
				button.Expires:Hide();
			end
		else
			button.Expires:Hide()
		end

		-- Update followers
		for i = 1, mission.numFollowers do
			button.MinimizedFollowers[i]:Show()
			if ( not parties[mission.missionID] ) then
				parties[mission.missionID] = {}
			end
			button.MinimizedFollowers[i]:setFollower(parties[mission.missionID][i]);
		end
		for i = mission.numFollowers + 1, #button.Followers do
			button.MinimizedFollowers[i].info = nil
			button.MinimizedFollowers[i]:Hide()
		end

		-- Rewards
		if ( mission.numRewards > 0 ) then
			local index = 1;
			for id, reward in pairs(mission.rewards) do
				if (not button.Rewards[index]) then
					button.Rewards[index] = CreateFrame("Frame", nil, button, "MultiPlannerListButtonRewardTemplate");
					button.Rewards[index]:SetPoint("RIGHT", button.Rewards[index-1], "LEFT", -4, 0);
				end
				button.Rewards[index]:setReward(reward)
				index = index + 1
			end
		end
		for i = (mission.numRewards + 1), #button.Rewards do
			button.Rewards[i]:Hide();
		end

		-- Mechanics
		updateCollapsedMissionMechanics(button)

		-- Party changes
		updateCollapsedMissionForParty(button)


		button.SelectedBox:Hide();
		module.setHighlightAlpha(button, 1)
	end
end
