local Lib = MissionMinder
Lib.callbacks = Lib.callbacks or LibStub("CallbackHandler-1.0"):New(Lib)

local OPTIONS = Lib.options;
local DEBUG = Lib.debug;
local module = Lib.multiplanner
local states = module.states

local L = LibStub("AceLocale-3.0"):GetLocale("MissionMinder");

local EXPANDED = states.BASE:new("EXPANDED")
states.EXPANDED = EXPANDED

function EXPANDED:new()
	local obj = {}
	setmetatable(obj, self)
	return obj
end

local function isEmpty(list, size, truevalue, falsevalue)
	list = list or {}
	size = size or #list
	truevalue = truevalue or true
	falsevalue = falsevalue or false

	for i = 1, size do
		if (list[i]) then
			return falsevalue
		end
	end

	return truevalue
end

local function adjustForExpandedDetails(button)
	local listscroll = GarrisonMissionFrame.MultiPlanner.MissionsList

	button:SetHeight(listscroll.collapsedButtonHeight + listscroll.expandedButtonHeight)
	button.IconBG:SetPoint("BOTTOMLEFT", button.Header, 0, 0)
	button.Header:Show()
	button.SelectT:Show()
	button.SelectB:Show()
	button.SelectTL:Show()
	button.SelectTR:Show()
	button.SelectBL:Show()
	button.SelectBR:Show()
	button.SelectedBox:Show();
	button.MissionTime:Show()
	button.MissionXP:Show()
	module.setHighlightAlpha(button, 0)

	button.Chance:SetPoint("CENTER", button.Header, "BOTTOMLEFT", 40, 14);
	button.Rewards[1]:ClearAllPoints()
	button.Rewards[1]:SetPoint("RIGHT", button.Header, "RIGHT", -9, 0)
	for i = 1, #button.Rewards do
		button.Rewards[i]:SetScale(.9)
	end

	button.Summary:Hide()
	button.RareOverlay:Hide()
	button.RareText:Hide()
	button.LocBG:Hide()
	button.Expires:Hide()

	button.MissionEnvIcon:Hide()
	for i = 1, #button.Mechanics do
		button.Mechanics[i]:Hide();
	end
	for i = 1, #button.MinimizedFollowers do
		button.MinimizedFollowers[i]:Hide()
	end
end




local function updateExpandedDetailsForParty_CheckCounter(button, counterID)
	for e = 1, #button.Enemies do
		local enemy = button.Enemies[e]
		for m = 1, #enemy.Mechanics do
			local mechanic = enemy.Mechanics[m]
			if ( counterID == mechanic.mechanicID and not mechanic.hasCounter ) then
				mechanic.hasCounter = true;
				mechanic.Check:Show();
				return;
			end
		end
	end
end

local function updateExpandedMissionForParty(button)
	local mission = button.info
	if ( not mission ) then
		return
	end

	local location, xp, environment, environmentDesc, environmentTexture, locPrefix, isExhausting, enemies = C_Garrison.GetMissionInfo(mission.missionID);
	local totalTimeString, totalTimeSeconds, isMissionTimeImproved, successChance, partyBuffs, isEnvMechanicCountered, xpBonus, materialMultiplier, goldMultiplier = C_Garrison.GetPartyMissionInfo(mission.missionID)

	-- TIME
	if ( isMissionTimeImproved ) then
		totalTimeString = GREEN_FONT_COLOR_CODE..totalTimeString..FONT_COLOR_CODE_CLOSE
	elseif ( totalTimeSeconds >= GARRISON_LONG_MISSION_TIME ) then
		totalTimeString = format(GARRISON_LONG_MISSION_TIME_FORMAT, totalTimeString)
	end
	button.Summary:SetFormattedText(PARENS_TEMPLATE, totalTimeString)
	button.MissionTime:SetFormattedText(GARRISON_MISSION_TIME_TOTAL, totalTimeString)

	-- SUCCESS CHANCE
	button.Chance:SetFormattedText(PERCENTAGE_STRING, successChance)

	-- MECHANICS
	for e = 1, #button.Enemies do
		local enemy = button.Enemies[e]
		for m = 1, #enemy.Mechanics do
			enemy.Mechanics[m].hasCounter = nil;
			enemy.Mechanics[m].Check:Hide();
		end
	end
	for i = 1, #button.Followers do
		local frame = button.Followers[i]
		local follower = frame.info
		if ( follower and C_Garrison.GetFollowerBiasForMission(mission.missionID, follower.followerID) > -1 ) then
			local abilities = C_Garrison.GetFollowerAbilities(follower.followerID)
			for a = 1, #abilities do
				local ability = abilities[a];
				for counterID, counterInfo in pairs(ability.counters) do
					updateExpandedDetailsForParty_CheckCounter(button, counterID);
				end
			end
		end
	end

	-- PARTY BUFFS
	local buffsFrame = button.PartyBuffs
	local buffCount = #partyBuffs;
	if ( buffCount == 0 ) then
		buffsFrame:Hide();
	else
		for i = 1, buffCount do
			local buff = buffsFrame.Buffs[i];
			if ( not buff ) then
				buff = CreateFrame("Frame", nil, buffsFrame, "GarrisonMissionPartyBuffTemplate");
				buff:SetPoint("LEFT", buffsFrame.Buffs[i - 1], "RIGHT", 8, 0);
			end
			buff.Icon:SetTexture(C_Garrison.GetFollowerAbilityIcon(partyBuffs[i]));
			buff.id = partyBuffs[i];
			buff:Show();
		end
		for i = buffCount + 1, #buffsFrame.Buffs do
			buffsFrame.Buffs[i]:Hide();
		end
		local width = buffCount * 28 + buffsFrame.BuffsTitle:GetWidth() + 40;
		buffsFrame:SetWidth(max(width, 160));
		buffsFrame:Show();
	end
	for i = buffCount + 1, #buffsFrame.Buffs do
		buffsFrame.Buffs[i]:Hide()
	end

	-- REWARDS
	GarrisonMissionPage_UpdateRewardQuantities(button, materialMultiplier, goldMultiplier);

	-- XP
	if ( xpBonus > 0 ) then
		button.MissionXP:SetFormattedText(GARRISON_MISSION_BASE_XP_PLUS, xp + xpBonus, xpBonus);
		button.MissionXP.hasBonusBaseXP = true;
	else
		button.MissionXP:SetFormattedText(GARRISON_MISSION_BASE_XP, xp);
		button.MissionXP.hasBonusBaseXP = false;
	end
end

function EXPANDED:getFollowerFrames(button)
	return button.Followers
end

function EXPANDED:missionEnter()
end

function EXPANDED:missionClick()
end

function EXPANDED:placerClick(placer, button)
	local buttons = GarrisonMissionFrame.MultiPlanner.MissionsList.buttons;
	local followers = button.Followers

	for i = 1, #followers do
		local frame = followers[i];
		if ( frame:IsShown() and frame:IsMouseOver() ) then
			GarrisonMissionPlannerFollowerFrame_OnReceiveDrag(frame);
		end
	end
end

local rateBack = 0.1;
local rateMid = 0.3;
local rateFore = 0.8;

function EXPANDED:animateButton(button, elapsed)
	local changeBack = rateBack/100 * elapsed;
	local changeMid = rateMid/100 * elapsed;
	local changeFore = rateFore/100 * elapsed;

	local backL, _, _, _, backR = button.LocBack:GetTexCoord();
	local midL, _, _, _, midR = button.LocMid:GetTexCoord();
	local foreL, _, _, _, foreR = button.LocFore:GetTexCoord();

	backL = backL + changeBack;
	backR = backR + changeBack;
	midL = midL + changeMid;
	midR = midR + changeMid;
	foreL = foreL + changeFore;
	foreR = foreR + changeFore;

	if (backL >= 1) then
		backL = backL - 1;
		backR = backR - 1;
	end
	if (midL >= 1) then
		midL = midL - 1;
		midR = midR - 1;
	end
	if (foreL >= 1) then
		foreL = foreL - 1;
		foreR = foreR - 1;
	end

	button.LocBack:SetTexCoord(backL, backR, 0, 1);
	button.LocMid:SetTexCoord (midL, midR, 0, 1);
	button.LocFore:SetTexCoord(foreL, foreR, 0, 1);
end

function EXPANDED:updateMissionDetails(button)
	local parties = module.parties
	local mission = button.info
	if ( mission and self.dirty ) then
		adjustForExpandedDetails(button)

		local location, xp, environment, environmentDesc, environmentTexture, locPrefix, isExhausting, enemies = C_Garrison.GetMissionInfo(mission.missionID);

		-- Title
		button.Title:SetWidth(0);
		button.Title:SetText(mission.name);
		button.Title:SetPoint("LEFT", 165, 0);

		-- Location
		if ( locPrefix ) then
			button.LocBack:SetAtlas("_"..locPrefix.."-Back", true);
			button.LocMid:SetAtlas ("_"..locPrefix.."-Mid", true);
			button.LocFore:SetAtlas("_"..locPrefix.."-Fore", true);

			button.LocBack:Show()
			button.LocMid:Show()
			button.LocFore:Show()
			button.LocBottomBlend:Show()
		else
			button.LocBack:Hide()
			button.LocMid:Hide()
			button.LocFore:Hide()
			button.LocBottomBlend:Hide()
		end

		-- Rare Mission
		if (mission.isRare) then
			button.IconBG:SetVertexColor(0, 0.012, 0.291, 0.5)
		else
			button.IconBG:SetVertexColor(0, 0, 0, 0.5)
		end

		-- Mission Type
		button.Icon:SetAtlas(mission.typeAtlas)

		-- Mission Level
		local showingItemLevel = false;
		if ( mission.level == GARRISON_FOLLOWER_MAX_LEVEL and mission.iLevel > 0 ) then
			showingItemLevel = true;
		end
		button.Level:SetText(showingItemLevel and mission.iLevel or mission.level);

		-- Success Chance
		local successChanceColor = isEmpty(parties[mission.missionID], mission.numFollowers, module.SUCCESS_CHANCE_DEFAULT_COLOR, module.SUCCESS_CHANCE_IN_USE_COLOR)
		button.Chance:SetTextColor(unpack(successChanceColor))

		-- Followers
		for i = 1, mission.numFollowers do
			button.Followers[i]:Show();
			if ( not parties[mission.missionID] ) then
				parties[mission.missionID] = {}
			end
			button.Followers[i]:setFollower(parties[mission.missionID][i]);
		end
		for i = mission.numFollowers + 1, #button.Followers do
			button.Followers[i].info = nil
			button.Followers[i]:Hide()
		end

		local shift = 25
		if ( mission.numFollowers == 1 ) then
			if ( #enemies == 1)  then
				button.Follower1:SetPoint("BOTTOMLEFT", shift + 82, 44);
			else
				button.Follower1:SetPoint("BOTTOMLEFT", shift + 22, 44);
			end
		elseif ( mission.numFollowers == 2 ) then
			button.Follower1:SetPoint("BOTTOMLEFT", shift + 108, 44);
		else
			button.Follower1:SetPoint("BOTTOMLEFT", shift + 22, 44);
		end

		-- Rewards
		if ( mission.numRewards > 0 ) then
			local index = 1;
			for id, reward in pairs(mission.rewards) do
				if (not button.Rewards[index]) then
					button.Rewards[index] = CreateFrame("Frame", nil, button, "MultiPlannerListButtonRewardTemplate");
					button.Rewards[index]:SetPoint("RIGHT", button.Rewards[index-1], "LEFT", -4, 0);
				end
				button.Rewards[index]:setReward(reward)
				index = index + 1
			end
		end
		for i = (mission.numRewards + 1), #button.Rewards do
			button.Rewards[i]:Hide();
		end

		-- Enemies and Mechanics
		local numEnemies = math.min(#enemies, #button.Enemies)
		if ( numEnemies == 1 ) then
			button.Enemy1:SetPoint("TOPLEFT", button.Header, "BOTTOMLEFT", 251, -30);
		elseif ( numEnemies == 2 ) then
			button.Enemy1:SetPoint("TOPLEFT", button.Header, "BOTTOMLEFT", 165, -30);
		else
			button.Enemy1:SetPoint("TOPLEFT", button.Header, "BOTTOMLEFT", 78, -30);
		end

		for i = 1, #enemies do
			if ( not button.Enemies[i] ) then
				break
			end
			button.Enemies[i]:setEnemy(enemies[i])
		end
		for i = #enemies + 1, #button.Enemies do
			button.Enemies[i]:Hide();
		end

		-- Party changes
		updateExpandedMissionForParty(button)
	end
end
