local Lib = MissionMinder
Lib.callbacks = Lib.callbacks or LibStub("CallbackHandler-1.0"):New(Lib)

local OPTIONS = Lib.options;
local DEBUG = Lib.debug;
local module = Lib.multiplanner
local states = module.states

local L = LibStub("AceLocale-3.0"):GetLocale("MissionMinder");



local PLANNER = states.BASE:new("PLANNER")
states.PLANNER = PLANNER

function PLANNER:new()
	local obj = {}
	setmetatable(obj, self)
	return obj
end

function PLANNER:initialize()
	-- Update mission list
	PlannerMissionList_Update()

	module.updateReadyMissionCount()
end

function PLANNER:destroy()
end

function PLANNER:tabShown()
	local activeFollowers = C_Garrison.GetNumActiveFollowers()
	local maxFollowers = C_Garrison.GetFollowerSoftCap()

	local availableFollowers = 1
	local smallestFollowerRequirement = 0
	if ( numMissions == 0 ) then
		module.showStatusOverlay(GARRISON_EMPTY_MISSION_LIST);
	elseif ( activeFollowers > maxFollowers ) then
		module.showStatusOverlay(GARRISON_PLANNER_TOO_MANY_ACTIVE_FOLLOWERS);
	elseif ( availableFollowers < smallestFollowerRequirement ) then
		module.showStatusOverlay(GARRISON_PLANNER_NOT_ENOUGH_FOLLOWERS_FOR_CURRENT_MISSIONS);
	else
		module.hideStatusOverlay()
	end

	-- Update mission list
	PlannerMissionList_Update()

	module.updateReadyMissionCount()
end

function PLANNER:startMissions()
	local missionsToSend = {}
	local missions = C_Garrison.GetAvailableMissions();
	for i = 1, #missions do
		local mission = missions[i];
		if ( #mission.followers == mission.numFollowers ) then
			tinsert(missionsToSend, mission);
		end
	end

	if ( #missionsToSend > 0 ) then
		module.setState(module.states.STARTING_MISSIONS, missionsToSend)
	end
end

function PLANNER:updateMissions()
	local multiplanner = GarrisonMissionFrame.MultiPlanner;
	local missions = C_Garrison.GetAvailableMissions();
	Garrison_SortMissions(missions);
	local numMissions = #missions;

	local scrollFrame = multiplanner.MissionsList;
	local offset = HybridScrollFrame_GetOffset(scrollFrame);
	local buttons = scrollFrame.buttons;
	local numButtons = #buttons;

	for i = 1, numButtons do
		local button = buttons[i];
		local index = offset + i;

		module.setMissionButton(button, missions, index)
		module.markForUpdate(button)
		if ( GarrisonMissionFrame.MultiPlanner.Overlay:IsShown() ) then
			button:Disable();
		else
			button:Enable();
		end
	end

	local expandedHeight = GarrisonMissionFrame.MultiPlanner.selectedMission and scrollFrame.expandedButtonHeight or 0
	local totalHeight = numMissions * scrollFrame.buttonHeight + expandedHeight;
	local displayedHeight = numButtons * scrollFrame.buttonHeight;
	HybridScrollFrame_Update(scrollFrame, totalHeight, displayedHeight);
end

function PLANNER:missionEnter()
end

function PLANNER:missionClick()
end
