local Lib = MissionMinder
Lib.callbacks = Lib.callbacks or LibStub("CallbackHandler-1.0"):New(Lib)

local OPTIONS = Lib.options;
local DEBUG = Lib.debug;
local module = Lib.multiplanner
local states = module.states

local L = LibStub("AceLocale-3.0"):GetLocale("MissionMinder");
local GARRISON_PLANNER_SENDING_MISSIONS = L["Sending missions..."]
local GARRISON_PLANNER_PROGRESS = L["%d of %d"];

local STARTING_MISSIONS = states.BASE:new("STARTING_MISSIONS", {
	index = 0,
	ticksRemaining = 0,
	timerTask = nil
})
states.STARTING_MISSIONS = STARTING_MISSIONS

local function startMission(mission)
	if ( mission ) then
		DEBUG("Sending mission (%d) \"%s\"", mission.missionID, mission.name);
		C_Garrison.StartMission(mission.missionID);
		PlaySound("UI_Garrison_CommandTable_MissionStart");
	end
end

local function animateSend(self, index)
	local MultiPlanner = GarrisonMissionFrame.MultiPlanner
	local offset = HybridScrollFrame_GetOffset(MultiPlanner.MissionsList);
	local buttons = MultiPlanner.MissionsList.buttons

	if ( index > #self.missions ) then
		return;
	end

	MultiPlanner.StartingOffscreenMissions:Hide()
	if ( index <= offset ) then
		local GlowBox = MultiPlanner.StartingOffscreenMissions.GlowBox
		GlowBox:ClearAllPoints()
		GlowBox:SetPoint("TOP",  MultiPlanner.MissionsList, 0, -5)
		GlowBox.ArrowDown:Hide()
		GlowBox.ArrowGlowDown:Hide()
		GlowBox.ArrowUp:Show()
		GlowBox.ArrowGlowUp:Show()
		GlowBox.BigText:SetText(GARRISON_PLANNER_SENDING_MISSIONS)
		GlowBox.SmallText:SetFormattedText(GARRISON_PLANNER_PROGRESS, index, #self.missions)
		GlowBox:SetHeight(12 + GlowBox.BigText:GetHeight() + 12 + GlowBox.SmallText:GetHeight() + 12)
		MultiPlanner.StartingOffscreenMissions:Show()
	elseif ( index >= offset + #buttons ) then
		local GlowBox = MultiPlanner.StartingOffscreenMissions.GlowBox
		GlowBox:ClearAllPoints()
		GlowBox:SetPoint("BOTTOM",  MultiPlanner.MissionsList, 0, 5)
		GlowBox.ArrowUp:Hide()
		GlowBox.ArrowGlowUp:Hide()
		GlowBox.ArrowDown:Show()
		GlowBox.ArrowGlowDown:Show()
		GlowBox.BigText:SetText(GARRISON_PLANNER_SENDING_MISSIONS)
		GlowBox.SmallText:SetFormattedText(GARRISON_PLANNER_PROGRESS, index, #self.missions)
		GlowBox:SetHeight(12 + GlowBox.BigText:GetHeight() + 12 + GlowBox.SmallText:GetHeight() + 12)
		MultiPlanner.StartingOffscreenMissions:Show()
	else
		buttons[index - offset].SendMission:Play()
	end
end

local function missionWasSent(mission)
	if ( mission) then
		local missions = C_Garrison.GetAvailableMissions();
		for i = 1, #missions do
			if( mission.missionID == missions[i].missionID ) then
				DEBUG("Mission %d still exists, retry", mission.missionID)
				return false
			end
		end
	end

	return true
end

local function findMissionToSend(self)
	if ( self.index > #self.missions ) then
		return nil, false
	else
		local mission = self.missions[self.index]
		if ( not mission or missionWasSent(mission) ) then -- Last mission was sent (or is nil); try the next
			self.index = self.index + 1
			return self.missions[self.index], true
		end

		return mission, false -- Last mission failed to send, try again
	end

end

local function startOneMission(self)

	local mission, firstTry = findMissionToSend(self)
	if ( mission ) then
		if ( firstTry ) then
			animateSend(self, self.index)
		end

		startMission(mission)

		if ( firstTry ) then
			Lib.callbacks:Fire("MISSION_STARTED", mission.missionID)
		end
	end

	self.ticksRemaining = self.ticksRemaining - 1
	if ( self.ticksRemaining <= 0 or self.index > #self.missions ) then
		self.timerTask:Cancel()
		self.ticksRemaining = 0
		self.timerTask = nil

		C_Timer.After(0.3, function()
			module.setState(states.PLANNER)
		end)

		return
	end
end

function STARTING_MISSIONS:new(missions)
	local obj = {}
	setmetatable(obj, self)

	obj.missions = missions
	return obj
end

function STARTING_MISSIONS:initialize()
	module.hideStatusOverlay()

	-- Show looping Starting Missions animation

	-- Prepare SendMisssions animation
	local buttons = GarrisonMissionFrame.MultiPlanner.MissionsList.buttons
	for i = 1, #buttons do
		buttons[i].SendMission:Stop()
		buttons[i].Overlay:Hide()
	end

	-- Update mission list
	GarrisonMissionFrame.MultiPlanner.MissionsList.scrollBar:SetValue(0)
	GarrisonMissionPlanner_DeselectMission()

	GarrisonMissionFrame.MultiPlanner.StartingOffscreenMissions:Hide()

	self.ticksRemaining = #self.missions * 20 + 1
	self.timerTask = C_Timer.NewTicker(0.1, function()
		startOneMission(self)
	end, ticksRemaining); -- allow for 2 seconds for each mission
end

function STARTING_MISSIONS:destroy()
	-- Stop looping Starting Missions animation

	-- Cleanup SendMisssions animation
	local buttons = GarrisonMissionFrame.MultiPlanner.MissionsList.buttons
	for i = 1, #buttons do
		buttons[i].SendMission:Stop()
		buttons[i].Overlay:Hide()
	end

	GarrisonMissionFrame.MultiPlanner.StartingOffscreenMissions:Hide()
end

function STARTING_MISSIONS:tabShown()
end

function STARTING_MISSIONS:updateMissions()
	local multiplanner = GarrisonMissionFrame.MultiPlanner;
	local missions = self.missions
	local numMissions = #missions;

	local scrollFrame = multiplanner.MissionsList;
	local offset = HybridScrollFrame_GetOffset(scrollFrame);
	local buttons = scrollFrame.buttons;
	local numButtons = #buttons;

	module.hideStatusOverlay()

	for i = 1, numButtons do
		local button = buttons[i];
		local index = offset + i;

		module.setMissionButton(button, missions, index)
		module.markForUpdate(button)

		button.SendMission:Stop()
		button.Overlay:Hide()
		if (self.index >= index ) then
			button.Overlay:Show()
			button.Overlay:SetAlpha(0.8)
		end
	end

	local expandedHeight = GarrisonMissionFrame.MultiPlanner.selectedMission and scrollFrame.expandedButtonHeight or 0
	local totalHeight = numMissions * scrollFrame.buttonHeight + expandedHeight;
	local displayedHeight = numButtons * scrollFrame.buttonHeight;
	HybridScrollFrame_Update(scrollFrame, totalHeight, displayedHeight);
end

function STARTING_MISSIONS:missionEnter()
end

function STARTING_MISSIONS:missionClient()
end
