local Lib = MissionMinder
Lib.callbacks = Lib.callbacks or LibStub("CallbackHandler-1.0"):New(Lib)

local OPTIONS = Lib.options;
local DEBUG = Lib.debug;
local module = Lib.multiplanner

local L = LibStub("AceLocale-3.0"):GetLocale("MissionMinder");

--Map of arrays to each missions party layout
--Used to determine the position that each follower is displayed within a party
-- parties[misisonID][1..3]
local parties = {}
module.parties = parties

module.SUCCESS_CHANCE_IN_USE_COLOR = { 0, 1, 0, 1 }
module.SUCCESS_CHANCE_DEFAULT_COLOR = { 0, 1, 0, 0.4 }



local counterLayout = {
	[1] = {
		[1] = { "LEFT", 64, -1 }
	},

	[2] = {
		[1] = { "LEFT", 64, -1 },
		[2] = { "LEFT", 1, "RIGHT", 16, 0 }
	},

	[3] = {
		[1] = { "LEFT", 64, -1 },
		[2] = { "LEFT", 1, "RIGHT", 16, 0 },
		[2] = { "LEFT", 2, "RIGHT", 16, 0 }
	}
}

local function applyCounterLayout(counter, layout, i)
	layout = layout[i]
	counter[i]:ClearAllPoints()

	if ( #layout == 3) then
		counter[i]:SetPoint(layout[1], layout[2], layout[3]);
	elseif ( #layout == 4) then
		counter[i]:SetPoint(layout[1], frame:GetParent(), layout[2], layout[3], layout[4]);
	elseif ( #layout == 5) then
		counter[i]:SetPoint(layout[1], counter[layout[2]], layout[3], layout[4], layout[5]);
	end
end


local function getFollowerMissionBiasRGB(missionID,followerID)
	local bias = C_Garrison.GetFollowerBiasForMission(missionID, followerID);

	if ( bias == -1 ) then -- Impossible
		return 1, 0.1, 0.1;
	elseif ( bias < 0 ) then -- Hard
		return 1, 0.5, 0.25;
	else -- Normal
		return 1, 1, 1;
	end
end

function GarrisonMissionPlannerButton_SetFollower(self, followerID)
	local mission = self:GetParent().info
	if ( mission and followerID ) then
		local follower = C_Garrison.GetFollowerInfo(followerID);
		self.info = follower;
		self.Name:SetText(follower.name);
		self.Name:Show();
		local showItemLevel;
		if ( mission.iLevel > 600 and follower.level == GarrisonMissionFrame.followerMaxLevel ) then
			self.PortraitFrame.LevelBorder:SetAtlas("GarrMission_PortraitRing_iLvlBorder");
			self.PortraitFrame.LevelBorder:SetWidth(70);
			showItemLevel = true;
		else
			self.PortraitFrame.LevelBorder:SetAtlas("GarrMission_PortraitRing_LevelBorder");
			self.PortraitFrame.LevelBorder:SetWidth(58);
			showItemLevel = false;
		end

		GarrisonMissionPlanner_SetFollowerPortrait(self.PortraitFrame, follower, showItemLevel);
		self.Class:SetAtlas(follower.classAtlas)
		self.Class:Show()
		self.PortraitFrame.Empty:Hide();

		self.PortraitFrame.Level:SetTextColor(getFollowerMissionBiasRGB(mission.missionID, follower.followerID));
	else
		self.info = nil;
		self.Class:Hide()
		self.Name:Hide();
		self.PortraitFrame.Empty:Show();
		self.PortraitFrame.LevelBorder:SetAtlas("GarrMission_PortraitRing_LevelBorder");
		self.PortraitFrame.LevelBorder:SetWidth(58);
		self.PortraitFrame.Level:SetText("");
	end

	Lib.planner.updateXPGains(mission, self);
	Lib.planner.updateLevelUp(mission, self);

	-- Counters
	for i = 1, #self.Counters do
		self.Counters[i]:ClearAllPoints()
	end

	local counters = module.getFollowerCounters(mission.missionID, followerID) or {}
	local layout = counterLayout[#counters]
	for i = 1, #counters do
		if (not self.Counters[i]) then
			self.Counters[i] = CreateFrame("Frame", nil, self, "GarrisonMissionAbilityLargeCounterTemplate");
		end
		local Counter = self.Counters[i];
		Counter.info = counters[i];
		Counter.info.showCounters = true;
		Counter.Icon:SetTexture(counters[i].icon);
		Counter.tooltip = counters[i].name;
		applyCounterLayout(self.Counters, layout, i)
		Counter:Show();

		if( counters[i].traitID ) then
			Counter.Border:Hide()
		else
			Counter.Border:Show()
		end
	end
	for i = (#counters + 1), #self.Counters do
		self.Counters[i]:Hide();
	end
end

function GarrisonMissionPlannerButton_SetMinimizedFollower(self, followerID)
	local mission = self:GetParent().info
	if ( mission and followerID ) then
		local follower = C_Garrison.GetFollowerInfo(followerID)
		self.info = follower
		self:SetAlpha(1)

		local showItemLevel;
		if ( mission.iLevel > 600 and follower.level == GarrisonMissionFrame.followerMaxLevel ) then
			self.LevelBorder:SetAtlas("GarrMission_PortraitRing_iLvlBorder");
			self.LevelBorder:SetWidth(70);
			showItemLevel = true;
		else
			self.LevelBorder:SetAtlas("GarrMission_PortraitRing_LevelBorder");
			self.LevelBorder:SetWidth(58);
			showItemLevel = false;
		end

		self.LevelBorder:Show()
		self.Level:SetTextColor(getFollowerMissionBiasRGB(mission.missionID, follower.followerID));
		self.Level:Show()
		GarrisonMissionPlanner_SetFollowerPortrait(self, follower, showItemLevel);

		local color = ITEM_QUALITY_COLORS[follower.quality];
		self.PortraitRingQuality:SetVertexColor(color.r, color.g, color.b);
	else
		self.info = nil;
		self:SetAlpha(0.7)

		self.PortraitRingQuality:SetVertexColor(0.3, 0.3, 0.3);
		GarrisonFollowerPortrait_Set(self.Portrait, nil);
		self.LevelBorder:SetAtlas("GarrMission_PortraitRing_LevelBorder");
		self.LevelBorder:SetWidth(58);
		self.Level:SetText("")
		self.LevelBorder:Hide()
		self.Level:Hide()
	end

	Lib.planner.updateLevelUp(mission, self);
end


function GarrisonMissionPlannerMinimizedFollower_OnMouseUp(self, button)
	if ( button == "RightButton" ) then
		if ( self.info ) then
			local mission = self:GetParent().info
			local follower = self.info
			GarrisonMissionPlanner_ClearFollower(mission, follower.followerID, self:GetID());
		end
	elseif ( not self.info ) then
		GarrisonMissionPlannerButton_OnClick(self:GetParent(), button)
	end
end

function GarrisonMissionPlannerButton_OnClick(self, button)
	local mission = self.info;
	if ( IsModifiedClick("CHATLINK") ) then
		local missionLink = C_Garrison.GetMissionLink(mission.missionID);
		if (missionLink) then
			ChatEdit_InsertLink(missionLink);
		end
		return;
	end

	PlaySound("UI_Garrison_CommandTable_SelectMission");
	if ( module.IsMissionSelected(mission) ) then
		GarrisonMissionPlanner_DeselectMission() -- Deselect misison
	else
		GarrisonMissionPlanner_SelectMission(mission) -- Select mission
	end
end

function GarrisonMissionPlannerMechanic_OnEnter(self)
	if ( self.info ) then
		GarrisonMissionMechanic_OnEnter(self)
	elseif ( self.environment ) then
		GameTooltip:SetOwner(self, "ANCHOR_BOTTOMRIGHT");
		GameTooltip:SetText(self.environment);
		GameTooltip:AddLine(self.environmentDesc, 1, 1, 1, 1);
		GameTooltip:Show();
	end
end

function GarrisonMissionPlannerMechanic_OnLeave(self)
	GarrisonMissionMechanicTooltip:Hide();
	GameTooltip:Hide()
end

function GarrisonMissionPlannerFollowerFrame_OnMouseUp(self, button)
	if ( button == "RightButton" ) then
		if ( not self.info and button == "RightButton") then
			GarrisonMissionPlannerButton_OnClick(self:GetParent(), button)
		else
			local mission = self:GetParent().info
			local follower = self.info
			GarrisonMissionPlanner_ClearFollower(mission, follower.followerID, self:GetID());
		end
	end
end

function module.clearFollowers(mission)
	for i = 1, mission.numFollowers do
		local party = parties[mission.missionID]
		if ( party and party[i] ) then
			C_Garrison.RemoveFollowerFromMission(mission.missionID, party[i])
			Lib.callbacks:Fire("FOLLOWER_REMOVED_FROM_MISSION", mission.missionID, i, "MultiPlanner")
		end
	end

	parties[mission.missionID] = {}
end

function GarrisonMissionPlannerButton_SetReward(self, reward)
	self.Quantity:Hide();
	self.IconBorder:Hide();
	self.itemID = nil;
	self.currencyID = nil;
	self.tooltip = nil;

	if (reward.itemID) then
		self.itemID = reward.itemID;
		GarrisonMissionFrame_SetItemRewardDetails(self);

		local _, _, quality = GetItemInfo(reward.itemID);
		if  ( quality and quality > LE_ITEM_QUALITY_COMMON and BAG_ITEM_QUALITY_COLORS[quality]) then
			self.IconBorder:Show();
			self.IconBorder:SetVertexColor(BAG_ITEM_QUALITY_COLORS[quality].r, BAG_ITEM_QUALITY_COLORS[quality].g, BAG_ITEM_QUALITY_COLORS[quality].b);
		end

		if ( reward.quantity > 1 ) then
			self.Quantity:SetText(reward.quantity);
			self.Quantity:Show();
		end
	else
		self.Icon:SetTexture(reward.icon);
		self.title = reward.title
		if (reward.currencyID and reward.quantity) then
			self.currencyID = reward.currencyID;
			self.currencyQuantity = reward.quantity
			if (reward.currencyID == 0) then
				self.tooltip = GetMoneyString(reward.quantity);
				self.Quantity:SetText(BreakUpLargeNumbers(math.floor(reward.quantity / COPPER_PER_GOLD)));
				self.Quantity:Show();
			else
				self.Quantity:SetText(reward.quantity);
				self.Quantity:Show();
			end
		else
			self.tooltip = reward.tooltip;
		end

		if ( reward.followerXP and reward.followerXP > 0 ) then
			self.Quantity:SetText(BreakUpLargeNumbers(reward.followerXP));
			self.Quantity:Show();
		end
	end
	self:Show();
end

function GarrisonMissionPlannerButton_SetEnemy(self, enemy)
	self.Name:SetText(enemy.name);
	GarrisonEnemyPortait_Set(self.PortraitFrame.Portrait, enemy.portraitFileDataID);
	local numMechs = 0;
	for id, mechanic in pairs(enemy.mechanics) do
		numMechs = numMechs + 1;
		if (not self.Mechanics[numMechs]) then
			self.Mechanics[numMechs] = CreateFrame("Button", nil, self, "GarrisonMissionEnemyLargeMechanicTemplate");
			self.Mechanics[numMechs]:SetPoint("LEFT", self.Mechanics[numMechs-1], "RIGHT", 16, 0);
			self.Mechanics[numMechs]:SetScript("OnClick", MissionMinder_FilterFollowersForCounter)
		end
		local Mechanic = self.Mechanics[numMechs];
		Mechanic.info = mechanic;
		Mechanic.Icon:SetTexture(mechanic.icon);
		Mechanic.mechanicID = id;
		Mechanic:Show();
	end
	self.Mechanics[1]:SetPoint("BOTTOM", (numMechs - 1) * -22, -16);
	for j=(numMechs + 1), #self.Mechanics do
		self.Mechanics[j]:Hide();
		self.Mechanics[j].mechanicID = nil;
		self.Mechanics[j].info = nil;
	end
	if ( numMechs > 1 ) then
		self.PortraitFrame.Elite:Show();
	else
		self.PortraitFrame.Elite:Hide();
	end
	self:Show();
end

function GarrisonMissionPlanner_SetFollowerPortrait(portraitFrame, followerInfo, showItemLevel)
	local color = ITEM_QUALITY_COLORS[followerInfo.quality];
	portraitFrame.PortraitRingQuality:SetVertexColor(color.r, color.g, color.b);
	portraitFrame.LevelBorder:SetVertexColor(color.r, color.g, color.b);
	if ( showItemLevel and followerInfo.level == GarrisonMissionFrame.followerMaxLevel ) then
		portraitFrame.Level:SetFormattedText(GARRISON_FOLLOWER_ITEM_LEVEL, followerInfo.iLevel);
	else
		portraitFrame.Level:SetText(followerInfo.level);
	end
	if ( followerInfo.displayID ) then
		GarrisonFollowerPortrait_Set(portraitFrame.Portrait, followerInfo.portraitIconID);
	end
end


local counterCache = {};
function module.getFollowerCounters(missionID, followerID)
	if ( not missionID or not followerID ) then
		return nil
	end

	if ( not counterCache[missionID]) then
		counterCache[missionID] = C_Garrison.GetBuffedFollowersForMission(missionID);
	end

	return counterCache[missionID][followerID];
end



function module.setHighlightAlpha(button, alpha)
	button.HighlightT:SetAlpha(alpha)
	button.HighlightB:SetAlpha(alpha)
	button.HighlightTL:SetAlpha(alpha)
	button.HighlightTR:SetAlpha(alpha)
	button.HighlightBL:SetAlpha(alpha)
	button.HighlightBR:SetAlpha(alpha)
	button.Highlight:SetAlpha(alpha)
end


function module.markForUpdate(button)
	button.dirty = true
end

function GarrisonMisisonPlannerButton_OnUpdate(self, elapsed)
	self.state:updateMissionDetails(self)
	self.state:animateButton(self, elapsed)
end

local function addFollowerToMission(event, missionID, index, followerID, source)
	local party = parties[missionID]
	if ( not party ) then
		party = {}
		parties[missionID] = party
	end

	if ( not index ) then -- general, add follower to mission; find first available
		for i = 1, #party + 1 do
			if ( not party[i] ) then
				index = i;
				break;
			end
		end
	end

	party[index] = followerID;
end

local function removeFollowerFromMission(event, missionID, index, source)
	if ( parties[missionID] ) then
		parties[missionID][index] = nil;
	end
end

-- Allow changes on the MissionPage to be reflected in the multiplanner
Lib.RegisterCallback("multiplanner", "FOLLOWER_ADDED_TO_MISSION", addFollowerToMission)
Lib.RegisterCallback("multiplanner", "FOLLOWER_REMOVED_FROM_MISSION", removeFollowerFromMission)
