-- Drag and drop controls for multiplanner
local L = LibStub("AceLocale-3.0"):GetLocale("MissionMinder");
local OPTIONS = MissionMinder.options;
local DEBUG = MissionMinder.debug;
local module = MissionMinder.multiplanner

local dragSource = nil
local sourceMission = nil
local sourceID = nil

local function showPlacer(info)
	GarrisonMissionPlanner_SetFollowerPortrait(GarrisonFollowerPlacer, info);
	GarrisonFollowerPlacer.info = info;
	local cursorX, cursorY = GetCursorPosition();
	local uiScale = GarrisonFollowerPlacer:GetEffectiveScale();
	GarrisonFollowerPlacer:SetPoint("TOP", UIParent, "BOTTOMLEFT", cursorX / uiScale, cursorY / uiScale + 24);
	GarrisonFollowerPlacer:Show();
	GarrisonFollowerPlacer:SetScript("OnUpdate", GarrisonFollowerPlacer_OnUpdate);
end

local function followerStopDrag(self)
	if (GarrisonFollowerPlacer:IsShown()) then
		GarrisonFollowerPlacerFrame:Show();
	end
end

local function followerStartDrag(self, button)
	dragSource, sourceMission, sourceID = nil, nil, nil

	if ( not GarrisonMissionFrame.MultiPlanner:IsShown() ) then
		return;
	end

	if ( self.info.status or not self.info.isCollected ) then
		return;
	end

	CloseDropDownMenus();
	showPlacer(self.info)
end

local function placerFrame_OnClick(self, button)
	local missionPage = GarrisonMissionFrame.MissionTab.MissionPage;

	if ( button == "LeftButton" ) then
		if ( missionPage:IsVisible() ) then
			module.GarrisonFollowerPlacerFrame_OnClick(self, button)
		else
			local buttons = GarrisonMissionFrame.MultiPlanner.MissionsList.buttons;
			for i = 1, #buttons do
				if ( buttons[i]:IsShown() and buttons[i]:IsMouseOver() ) then
					buttons[i].state:placerClick(self, buttons[i])
					break;
				end
			end
		end
	end

	GarrisonMissionFrame_ClearMouse();
end

-- If the dragSource gets hidden then OnDragStop won't be called causing dragging to break.
--To address this we manually call OnDragStop when dragSource is hidden
function GarrisonMissionPlanner_ShowPlacerFrameIfDragging(self)
	if ( dragSource and dragSource == self ) then
		self:GetScript("OnDragStop")()
	end
end

function GarrisonMissionPlannerFollowerFrame_OnDragStart(self)
	local mission = self:GetParent().info
	local follower = self.info
	if ( not follower ) then
		return;
	end


	showPlacer(follower)
	GarrisonMissionPlanner_ClearFollower(mission, follower.followerID, self:GetID());

	dragSource, sourceMission, sourceID = self, mission, self:GetID()
end

function GarrisonMissionPlannerFollowerFrame_OnDragStop(self)
	if (GarrisonFollowerPlacer:IsShown()) then
		GarrisonFollowerPlacerFrame:Show();
	end
end

function GarrisonMissionPlannerFollowerFrame_OnReceiveDrag(self)
	if ( GarrisonFollowerPlacer:IsVisible() and GarrisonFollowerPlacer.info ) then
		local targetFollower, targetMission, targetID = self.info, self:GetParent().info, self:GetID()
		local sourceFollower = GarrisonFollowerPlacer.info

		GarrisonMissionPlanner_SetFollower(targetMission, sourceFollower.followerID, targetID);
		GarrisonMissionFrame_ClearMouse();

		-- A follower was already placed here, and the follower that is being placed came from another frame.
		--Lets switch them!
		if ( targetFollower ) then
			GarrisonMissionPlanner_SetFollower(sourceMission, targetFollower.followerID, sourceID);
		end
	end
	dragSource, sourceMission, sourceID = nil, nil, nil
end

function GarrisonMissionPlannerMinimizedFollower_OnDragStart(self)
	local mission = self:GetParent().info
	local follower = self.info
	if ( not follower ) then
		return;
	end


	showPlacer(follower)
	GarrisonMissionPlanner_ClearFollower(mission, follower.followerID, self:GetID());

	dragSource, sourceMission, sourceID = self, mission, self:GetID()
end

function GarrisonMissionPlannerMinimizedFollower_OnDragStop(self)
	if (GarrisonFollowerPlacer:IsShown()) then
		GarrisonFollowerPlacerFrame:Show();
	end
end

function GarrisonMissionPlannerMinimizedFollower_OnReceiveDrag(self)
	local frame = self:GetParent().Followers[self:GetID()]
	if ( GarrisonFollowerPlacer:IsVisible() and GarrisonFollowerPlacer.info ) then
		local targetFollower, targetMission, targetID = self.info, self:GetParent().info, self:GetID()
		local sourceFollower = GarrisonFollowerPlacer.info

		GarrisonMissionPlanner_SetFollower(targetMission, sourceFollower.followerID, targetID);
		GarrisonMissionFrame_ClearMouse();

		-- A follower was already placed here, and the follower that is being placed came from another frame.
		--Lets switch them!
		if ( targetFollower ) then
			GarrisonMissionPlanner_SetFollower(sourceMission, targetFollower.followerID, sourceID);
		end
	end
	dragSource, sourceMission, sourceID = nil, nil, nil
end

function module.dragndrop_init()
	local followerButtons = GarrisonMissionFrame.FollowerList.listScroll.buttons;
	for i = 1, #followerButtons do
		followerButtons[i]:HookScript("OnDragStart", followerStartDrag);
		followerButtons[i]:HookScript("OnDragStop", followerStopDrag);
	end

	module.GarrisonFollowerPlacerFrame_OnClick = GarrisonFollowerPlacerFrame_OnClick
	GarrisonFollowerPlacerFrame:SetScript("OnClick", placerFrame_OnClick);
end
