local Lib = MissionMinder
Lib.callbacks = Lib.callbacks or LibStub("CallbackHandler-1.0"):New(Lib)
local L = LibStub("AceLocale-3.0"):GetLocale("MissionMinder");
local OPTIONS = Lib.options;
local DEBUG = Lib.debug;

local module = Lib.multiplanner;

GARRISON_PLANNER = "|cff00ff00*BETA*|r " .. L["MultiPlanner"];
GARRISON_PLANNER_START_MISSIONS = L["Start Missions"];
GARRISON_PLANNER_TOO_MANY_ACTIVE_FOLLOWERS = L["Too many active followers"];
GARRISON_PLANNER_NOT_ENOUGH_FOLLOWERS_FOR_CURRENT_MISSIONS = L["Not enough available followers for current missions"];
GARRISON_PLANNER_NO_MISSION_SELECTED = L["No mission has been selected."];

local MULTIPLANNER_TAB = 3

local DEFAULT_MISSION_COST_COLOR = { 1, 1, 1, 1 }
local NOT_ENOUGH_MISSION_COST_COLOR = { 1, 0, 0, 1 }

local missionsReadyToSend = 0;
local readyMissionCost = 0
local missionFollowerCounts = {};

local startMissionTask = nil;
local missionsToSend = {};
local startMissionIndex = 0;


local EXPANDED_STATE = module.states.EXPANDED:new()
local COLLAPSED_STATE = module.states.COLLAPSED:new()

local importantTraits = {
	[256] = true, -- Treasure Hunter
	[79] = true,  -- Scavenger
	[221] = true, -- Epic Mount
	[250] = true, -- Speed of Light
	[248] = true  -- Mentor
}

module.state = nil

local nopState = setmetatable({ name = "NOP" }, {
	__index = function(self, key)
		return function(...) end
	end
})

function module.getState()
	return module.state or nopState
end

function module.setState(state, ...)
	if ( module.state ) then
		module.state:destroy()
	end
	module.state = state:new(...)
	module.state:initialize()
	DEBUG("State changed to %s", module.state.name)
end

function module.getSelectedMission()
	local MissionPage = GarrisonMissionFrame.MissionTab.MissionPage
	local MultiPlanner = GarrisonMissionFrame.MultiPlanner

	if ( MultiPlanner:IsShown() and MultiPlanner.selectedMission ) then
		return MultiPlanner, MultiPlanner.selectedMission, MultiPlanner.mentorLevel or 0, MultiPlanner.mentorILevel or 0
	elseif ( MissionPage:IsVisible() and MissionPage.missionInfo ) then
		return MissionPage, MissionPage.missionInfo, MissionPage.mentorLevel or 0, MissionPage.mentorItemLevel or 0
	end

	return nil, nil, 0, 0
end

function module.setMissionButton(button, missions, index)
	if ( index <= #missions) then
		local mission = missions[index];
		button.id = index;
		button.info = mission;
		button.state = module.IsMissionSelected(mission) and EXPANDED_STATE or COLLAPSED_STATE
		button:Show();
	else
		button.id = nil
		button.info = nil
		button:Hide();
	end
end

function module.hideStatusOverlay()
	local self = GarrisonMissionFrame.MultiPlanner;
	self.Overlay:Hide();
end

function module.showStatusOverlay(text, ...)
	local self = GarrisonMissionFrame.MultiPlanner;
	self.Overlay.Status:SetText(string.format(text, ...));
	self.Overlay:Show();
end

function PlannerMissionList_Update()
	module.getState():updateMissions()
end


local function updateStartMissionsButton()
	local MissionCost = GarrisonMissionFrame.MultiPlanner.MissionCost.Cost
	MissionCost:SetText(readyMissionCost)
	MissionCost:SetTextColor(unpack(DEFAULT_MISSION_COST_COLOR))

	local _, garrisonResources = GetCurrencyInfo(GARRISON_CURRENCY)

	local errorString = nil;
	if ( missionsReadyToSend == 0 ) then
		errorString = GARRISON_PARTY_NOT_FULL_TOOLTIP
	elseif ( garrisonResources < readyMissionCost ) then
		errorString = GARRISON_NOT_ENOUGH_MATERIALS_TOOLTIP
		MissionCost:SetTextColor(unpack(NOT_ENOUGH_MISSION_COST_COLOR))
	end

	local StartMissions = GarrisonMissionFrame.MultiPlanner.StartMissions
	if ( errorString ) then
		StartMissions:SetText(GARRISON_PLANNER_START_MISSIONS)
		StartMissions.tooltip = errorString
		StartMissions.Flash:Hide()
		StartMissions.FlashAnim:Stop()
		StartMissions:Disable()
	else
		StartMissions:SetFormattedText("%s (%d)", GARRISON_PLANNER_START_MISSIONS, missionsReadyToSend)
		StartMissions.tooltip = nil
		StartMissions.Flash:Show()
		StartMissions.FlashAnim:Play()
		StartMissions:Enable()
	end
end

local function increaseReadyMissions(mission)
	missionsReadyToSend = missionsReadyToSend + 1
	readyMissionCost = readyMissionCost + mission.cost
	updateStartMissionsButton()
end

local function decreaseReadyMissions(mission)
	missionsReadyToSend = missionsReadyToSend - 1
	readyMissionCost = readyMissionCost - mission.cost
	updateStartMissionsButton()
end

function GarrisonMissionMultiPlanner_OnShow(self)
	GarrisonMissionFrame.FollowerList.showCounters = true;
	GarrisonMissionFrame.FollowerList.canExpand = true;
	GarrisonMissionFrame.FollowerList.showUncollected = false;

	module.getState():tabShown()
end

function GarrisonMissionMultiPlanner_OnHide(self)
	GarrisonMissionFrame.FollowerList.showCounters = false;
	GarrisonMissionFrame.FollowerList.canExpand = false;
	GarrisonMissionFrame.FollowerList.showUncollected = false;

	module.getState():tabHidden()
end

function GarrisonMissionPlannerStartMissions_OnClick(self, button)
	local state = module.getState():startMissions()
end

local function clearAllParties()
	local missions = C_Garrison.GetAvailableMissions();
	for i = 1, #missions do
		local mission = missions[i];
		module.clearFollowers(mission);
	end
end


function module.updateReadyMissionCount()
	missionsReadyToSend = 0;
	readyMissionCost = 0
	missionFollowerCounts = {};
	updateStartMissionsButton()

	local missions = C_Garrison.GetAvailableMissions();
	for i = 1, #missions do
		local mission = missions[i]
		missionFollowerCounts[mission.missionID] = mission.numFollowers - #mission.followers
		if ( missionFollowerCounts[mission.missionID] == 0 ) then
			increaseReadyMissions(mission)
		end
	end
end


function GarrisonMissionPlanner_ClearFollower(mission, followerID, id)
	if (mission and followerID) then
		PlaySound("UI_Garrison_CommandTable_UnassignFollower");
		C_Garrison.RemoveFollowerFromMission(mission.missionID, followerID)
		Lib.callbacks:Fire("FOLLOWER_REMOVED_FROM_MISSION", mission.missionID, id, "MultiPlanner")

		if ( module.IsMissionSelected(mission) ) then
			-- Update counters (incase removing this follower affects counter opportunities; racial preferences)
			GarrisonMissionFrame.followerCounters = C_Garrison.GetBuffedFollowersForMission(mission.missionID);
			GarrisonMissionFrame.followerTraits = C_Garrison.GetFollowersTraitsForMission(mission.missionID);
		end
		GarrisonFollowerList_DirtyList(GarrisonMissionFrame.FollowerList)
		GarrisonFollowerList_UpdateFollowers(GarrisonMissionFrame.FollowerList);

		if ( missionFollowerCounts[mission.missionID] == 0 ) then
			decreaseReadyMissions(mission)
		end
		missionFollowerCounts[mission.missionID] = missionFollowerCounts[mission.missionID] + 1

		PlannerMissionList_Update()
	end
end


function module.IsMissionSelected(mission)
	local selectedMission = GarrisonMissionFrame.MultiPlanner.selectedMission
	return selectedMission and mission and selectedMission.missionID == mission.missionID
end

function GarrisonMissionPlanner_DeselectMission()
	GarrisonMissionFrame.MultiPlanner.selectedMission = nil
	Lib.callbacks:Fire("MISSION_SELECTED", nil)
end

function GarrisonMissionPlanner_SelectMission(mission)
	if ( mission ) then
		GarrisonMissionFrame.MultiPlanner.selectedMission = mission
		Lib.callbacks:Fire("MISSION_SELECTED", mission.missionID)
	end
end


function GarrisonMissionPlanner_AddFollower(mission, followerID)
	GarrisonMissionPlanner_SetFollower(mission, followerID, nil)
end

function GarrisonMissionPlanner_SetFollower(mission, followerID, id)
	if ( mission and followerID ) then
		if ( id and module.parties[mission.missionID][id] ) then -- Follower already set, clear it before setting a new one
			GarrisonMissionPlanner_ClearFollower(mission, module.parties[mission.missionID][id], id)
		end

		if ( not C_Garrison.AddFollowerToMission(mission.missionID, followerID) ) then
			return;
		end
		Lib.callbacks:Fire("FOLLOWER_ADDED_TO_MISSION", mission.missionID, id, followerID, "MultiPlanner")

		PlaySound("UI_Garrison_CommandTable_AssignFollower");
		if ( module.IsMissionSelected(mission) ) then
			-- Update counters (incase adding this follower creates new counter opportunities; racial preferences)
			GarrisonMissionFrame.followerCounters = C_Garrison.GetBuffedFollowersForMission(mission.missionID);
			GarrisonMissionFrame.followerTraits = C_Garrison.GetFollowersTraitsForMission(mission.missionID);
		end
		GarrisonFollowerList_UpdateFollowers(GarrisonMissionFrame.FollowerList);

		missionFollowerCounts[mission.missionID] = missionFollowerCounts[mission.missionID] - 1
		if ( missionFollowerCounts[mission.missionID] == 0 ) then
			increaseReadyMissions(mission)
		end

		PlannerMissionList_Update()
	end
end

local function isTraitImportant(trait)
	return trait.isTrait and importantTraits[trait.id]
end

local function getImportantCounters()
	local abilityCounters, traitCounters = {}, {}

	local followers = GarrisonMissionFrame.FollowerList.followers or C_Garrison.GetFollowers()
	for i = 1, #followers do
		local followerID = followers[i].followerID

		local followerAbilityCounters, followerTraitCounters = {}, {}
		abilityCounters[followerID] = followerAbilityCounters
		traitCounters[followerID] = followerTraitCounters

		local abilities = C_Garrison.GetFollowerAbilities(followerID)
		for i = 1, #abilities do
			local ability = abilities[i]
			if ( ability.isTrait ) then
				if ( OPTIONS.ShowImportantTraits() and isTraitImportant(ability) ) then
					tinsert(followerTraitCounters, {
						traitID = ability.id,
						icon = ability.icon
					})
				end
			elseif ( OPTIONS.ShowImportantAbilities() ) then
				for counterID, counter in pairs(ability.counters) do
					tinsert(followerAbilityCounters, {
						counterIcon = ability.icon,
						name = counter.name,
						counterName = ability.name,
						icon = counter.icon,
						description = counter.description
					})
				end
			end
		end
	end

	return abilityCounters, traitCounters
end

local function updateFollowerListForMission(event, missionID)
	if ( missionID ) then
		GarrisonMissionFrame.followerCounters = C_Garrison.GetBuffedFollowersForMission(missionID)
		GarrisonMissionFrame.followerTraits = C_Garrison.GetFollowersTraitsForMission(missionID)
	else
		GarrisonMissionFrame.followerCounters, GarrisonMissionFrame.followerTraits = getImportantCounters()
	end

	if ( GarrisonMissionFrame.FollowerList.followers ) then
		GarrisonFollowerList_UpdateFollowers(GarrisonMissionFrame.FollowerList)
	end
end


local function selectTab(id)
	local CompleteDialog = GarrisonMissionFrame.MissionTab.MissionList.CompleteDialog
	if( id == MULTIPLANNER_TAB ) then -- Planner
		CompleteDialog:SetParent(GarrisonMissionFrame.MultiPlanner)
		PanelTemplates_SetTab(GarrisonMissionFrame, id);

		GarrisonMissionFrame.MissionTab:Hide();
		GarrisonMissionFrame.FollowerTab:Hide();
		GarrisonMissionFrame.MultiPlanner:Show()


		if ( GarrisonMissionFrame.FollowerList:IsShown() ) then
			GarrisonFollowerList_UpdateFollowers(GarrisonMissionFrame.FollowerList);
		else
			GarrisonMissionFrame.FollowerList:Show();
		end
		GarrisonMissionFrame.TitleText:SetText(L["Garrison Mission Planner"]);

		if ( UIDropDownMenu_GetCurrentDropDown() == GarrisonFollowerOptionDropDown ) then
			CloseDropDownMenus();
		end
	else -- Mission or Follower
		CompleteDialog:SetParent(GarrisonMissionFrame.MissionTab.MissionList)
		GarrisonMissionFrame.MultiPlanner:Hide()
		module.GarrisonMissionFrame_SelectTab(id)
	end
end

local function selectMissionsTab()
	if (PanelTemplates_GetSelectedTab(GarrisonMissionFrame) == MULTIPLANNER_TAB) then
		if ( GarrisonMissionFrame.MissionTab.MissionPage:IsShown() ) then
			GarrisonMissionPage_Close()
		end
		GarrisonMissionFrame_SelectTab(1); -- Missions
	end
end

local function onMissionFinished(onShow)
	local self = GarrisonMissionFrame;
	if ( self.MissionComplete:IsShown() ) then
		return;
	end

	if ( #self.MissionComplete.completeMissions > 0 and self:IsShown() ) then
		if ( PanelTemplates_GetSelectedTab(self) == MULTIPLANNER_TAB ) then
			GarrisonMissionFrame_SelectTab(1); -- Missions
		end
	end
end


local function disableMultiPlanner()
	PanelTemplates_DisableTab(GarrisonMissionFrame, MULTIPLANNER_TAB)
end

local function enableMultiPlanner()
	PanelTemplates_EnableTab(GarrisonMissionFrame, MULTIPLANNER_TAB)
end

local function getTopButton(self, offset)
	local selectedMission = GarrisonMissionFrame.MultiPlanner.selectedMission or {}
	local missions = C_Garrison.GetAvailableMissions()
	Garrison_SortMissions(missions);

	local totalHeight = 0;
	for i = 1, #missions do
		local height;
		if ( missions[i].missionID == selectedMission.missionID ) then
			height = self.buttonHeight + self.expandedButtonHeight
		else
			height = self.buttonHeight;
		end

		totalHeight = totalHeight + height;
		if ( totalHeight > offset ) then
			return i - 1, height + offset - totalHeight;
		end
	end

	return #missions, 0;
end

local function clearMissionPagePartyFrame(frame)
	frame.info = nil;
	frame.Name:Hide();
	if (frame.Class) then
		frame.Class:Hide();
	end
	frame.PortraitFrame.Empty:Show();
	frame.PortraitFrame.LevelBorder:SetAtlas("GarrMission_PortraitRing_LevelBorder");
	frame.PortraitFrame.LevelBorder:SetWidth(58);
	frame.PortraitFrame.Level:SetText("");
	frame.PortraitFrame.Caution:Hide();

	for i = 1, #frame.Counters do
		frame.Counters[i]:Hide();
	end
end

local function closeMissionPageOnSend(event, missionID)
	local MultiPlanner = GarrisonMissionFrame.MultiPlanner
	local MissionPage = GarrisonMissionFrame.MissionTab.MissionPage

	if ( MultiPlanner:IsShown() and MissionPage.missionInfo and MissionPage.missionInfo.missionID == missionID ) then
		MissionPage:Hide();
		MissionPage.missionInfo = nil;
		for i = 1, #MissionPage.Followers do
			clearMissionPagePartyFrame(MissionPage.Followers[i])
		end
	end
end

local function cleanupData(self)
	GarrisonMissionFrame.FollowerList.followers = nil
	GarrisonMissionFrame.followerCounters = nil
	GarrisonMissionFrame.followerTraits = nil
end

function PlannerMissionList_OnLoad(self)
end

local function initialize()
	local self = GarrisonMissionFrame.MultiPlanner.MissionsList
	self.update = PlannerMissionList_Update
	self.dynamic = function(offset) return getTopButton(self, offset); end
	HybridScrollFrame_CreateButtons(self, "MultiPlannerListButtonTemplate", 5, -4, nil, nil, nil, -4)
	PlannerMissionList_Update()

	local self = GarrisonMissionFrame.MultiPlanner
	GarrisonMissionFrame:HookScript("OnHide", clearAllParties);
	GarrisonMissionFrame:HookScript("OnHide", selectMissionsTab);
	GarrisonMissionFrame:HookScript("OnHide", cleanupData)

	for _,button in pairs(self.MissionsList.buttons) do
		button:HookScript("OnClick", PlannerMissionList_Update);
	end

	-- Add Planner tab
	module.GarrisonMissionFrame_SelectTab = GarrisonMissionFrame_SelectTab
	GarrisonMissionFrame_SelectTab = selectTab

	-- Disable MultiPlanner on MissionComplete show
	local missionComplete = GarrisonMissionFrame.MissionComplete
	missionComplete:HookScript("OnShow", disableMultiPlanner);
	missionComplete:HookScript("OnHide", enableMultiPlanner);

	-- Switch to MissionTab on GARRISON_MISSION_FINISHED event to display the CompleteDialog
	hooksecurefunc("GarrisonMissionFrame_CheckCompleteMissions", onMissionFinished)


	PanelTemplates_SetNumTabs(GarrisonMissionFrame, 3);
	PanelTemplates_UpdateTabs(GarrisonMissionFrame);

	module.dragndrop_init()

	-- MissionSelected
	Lib.RegisterCallback("multiplanner", "MISSION_SELECTED", PlannerMissionList_Update);
	Lib.RegisterCallback("multiplanner.followerList", "MISSION_SELECTED", updateFollowerListForMission);

	-- MissionClosed
	Lib.RegisterCallback(Lib, "MissionClosed", PlannerMissionList_Update);

	-- MultiPlanner should update followerCounters and Traits for its selected mission
	GarrisonMissionFrame.MultiPlanner:HookScript("OnShow", function(self)
		local mission = GarrisonMissionFrame.MultiPlanner.selectedMission or {}
		updateFollowerListForMission(nil, mission.missionID)
		GarrisonFollowerList_DirtyList(GarrisonMissionFrame.FollowerList)
		GarrisonFollowerList_UpdateFollowers(GarrisonMissionFrame.FollowerList)
	end)

	-- Close MissionPage if we send its mission
	Lib.RegisterCallback("multiplanner", "MISSION_STARTED", closeMissionPageOnSend)

	-- Initialize state
	GarrisonMissionFrame.MultiPlanner.selectedMission = nil
	module.setState(module.states.PLANNER)
end

local events = Lib.events
events:RegisterEvent("multiplanner", "ADDON_LOADED", initialize)
events:RegisterEvent("multiplanner", "GARRISON_MISSION_LIST_UPDATE", PlannerMissionList_Update);
